@@ -12,6 +12,13 @@ test_that("1 sample t test of multiple hypotheses correctly evaluated", {
     all.equal(unname(c(BF1$PHP_exploratory)),unname(BF1$PHP_confirmatory))
 )})
 
+ttest1 <- t_test(therapeutic,mu=5)
+BF1 <- BF(x=ttest1,hypothesis="mu=5;mu<5",complement=FALSE)
+test_that("1 sample t test of multiple hypotheses correctly evaluated", {
+  expect_equal(
+    c(0.3523759,0.6476241),unname(BF1$PHP_confirmatory),tol=.01
+  )})
+
 # test if one-sided PMP is same as one-sided p-value
 ttest2 <- t_test(therapeutic,mu=5,alternative="less")
 BF2 <- BF(x=ttest2,hypothesis="mu>5")
